﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using Rhino;
using Rhino.Collections;
using Rhino.FileIO;
using Rhino.PlugIns;

namespace TestMatthieu
{
  public class TestMatthieuPlugIn : PlugIn
  {
    private const uint TCODE_ANONYMOUS_CHUNK = 0x40000000 | 0x8000 | 0x0000; // from openNURBS
    private const int DOC_DATA_MAJOR_VERSION = 1;
    private const int DOC_DATA_MINOR_VERSION = 0;

    private ArchivableDictionary m_dictionary;

    public TestMatthieuPlugIn()
    {
      Instance = this;
      m_dictionary = new ArchivableDictionary(DOC_DATA_MAJOR_VERSION, Name);
    }

    /// <summary>
    /// Gets the only instance of the TestMatthieuPlugIn plug-in.
    /// </summary>
    public static TestMatthieuPlugIn Instance
    {
      get; private set;
    }

    public void FillDictionary()
    {
      if (null == m_dictionary)
        m_dictionary = new ArchivableDictionary(DOC_DATA_MAJOR_VERSION, Name);

      var deck_rooms = new string[] { "This", "is", "a", "test" };
      m_dictionary.Set("DeckRooms", deck_rooms);
      m_dictionary.Set("DeckName", "This is a test");
    }

    public void PrintDictionary()
    {
      if (m_dictionary.ContainsKey("DeckRooms"))
      {
        if (m_dictionary["DeckRooms"] is IEnumerable deck_rooms)
        {
          RhinoApp.WriteLine("DeckRooms:");
          foreach (var str in (IEnumerable) deck_rooms)
          {
            RhinoApp.WriteLine("  {0}", str);
          }
        }
      }
      if (m_dictionary.ContainsKey("DeckName"))
      {
        var str = m_dictionary["DeckName"] as string;
        RhinoApp.WriteLine("DeckName:");
        RhinoApp.WriteLine("  {0}", str);
      }

    }

    /// <summary>
    /// Called when the plug-in is being loaded.
    /// </summary>
    protected override LoadReturnCode OnLoad(ref string errorMessage)
    {
      // Add an event handler so we know when documents are closed.
      RhinoDoc.CloseDocument += OnCloseDocument;
      return LoadReturnCode.Success;
    }

    /// <summary>
    /// OnCloseDocument event handler.
    /// </summary>
    private void OnCloseDocument(object sender, DocumentEventArgs e)
    {
      m_dictionary.Clear();
    }

    /// <summary>
    /// ShouldCallWriteDocument override
    /// </summary>
    protected override bool ShouldCallWriteDocument(FileWriteOptions options)
    {
      return m_dictionary.Count > 0 &&
             !options.WriteGeometryOnly && 
             !options.WriteSelectedObjectsOnly;
    }

    /// <summary>
    /// WriteDocument override
    /// </summary>
    protected override void WriteDocument(RhinoDoc doc, BinaryArchiveWriter archive, FileWriteOptions options)
    {
      // Begin writing a chunk
      var rc = archive.BeginWrite3dmChunk(TCODE_ANONYMOUS_CHUNK, DOC_DATA_MAJOR_VERSION, DOC_DATA_MINOR_VERSION);
      if (!rc)
        return;

      // Write data like this:
      for (; ; )
      {
        // Write version 1.0 fields
        archive.WriteDictionary(m_dictionary);
        if (archive.WriteErrorOccured) break;

        // If you add data members to your document user data after you 
        // have released a product, simply increment DOC_DATA_MINOR_VERSION
        // by one, and write the new information.  This way you
        // can enhance your user data over several versions of
        // your product without breaking file IO.  It is CRITICAL
        // that once you write something to a customer's .3dm
        // file, you continue to write it the same way for all
        // future versions.

        // The code in the comment below demonstrates how to
        // correctly write information added in later releases
        // of your product.

        // Write version 1.1 fields
        // archive.WriteSomethingNew(...);
        // if (archive.WriteErrorOccured) break;
        // archive.WriteSomethingElseNew(...);
        // if (archive.WriteErrorOccured) break;

        // Write version 1.2 fields
        // archive.WriteAnotherSomethingNew(...);
        // if (archive.WriteErrorOccured) break;
        // archive.WriteAnotherSomethingElseNew(...);
        // if (archive.WriteErrorOccured) break;

        break;
      }

      // If BinaryArchiveWriter.BeginWrite3dmChunk returns true,
      // then BinaryArchiveWriter.EndWrite3dmChunl must be called
      // even if a write operation failed.
      rc = archive.EndWrite3dmChunk();

      if (!rc)
      {
        Debug.WriteLine("Error writing document user data.");
      }
    }

    protected override void ReadDocument(RhinoDoc doc, BinaryArchiveReader archive, FileReadOptions options)
    {
      // If our document user data is found, we MUST read it

      // Begin reading a chunk
      var rc = archive.BeginRead3dmChunk(TCODE_ANONYMOUS_CHUNK, out var major_version, out var minor_version);
      if (!rc)
        return;

      var keep = !options.ImportMode;

      // Read data like this:
      for (; ; )
      {
        rc = (1 == major_version);
        if (!rc) break;

        // Read version 1.0 fields
        var dictionary = archive.ReadDictionary();
        if (archive.ReadErrorOccured) break;

        if (dictionary.Count > 0 && keep)
        {
          if (m_dictionary.Count > 0)
            m_dictionary.Clear();
          m_dictionary.AddContentsFrom(dictionary);

          PrintDictionary();
        }

        // The code in the comment below demonstrates how to
        // correctly read information added in later releases
        // of your product.

        //if (minor_version >= 1)
        //{
        //  // Read version 1.1 and greater fields
        //  archive.ReadSomethingNew(...);
        //  if (archive.ReadErrorOccured) break;
        //  rc = archive.ReadSomethingElseNew(...);
        //  if (archive.ReadErrorOccured) break;

        //  if (minor_version >= 2)
        //  {
        //    // Read version 1.2 and greater fields
        //    rc = archive.ReadAnotherSomethingNew(...);
        //    if (archive.ReadErrorOccured) break;
        //    rc = archive.ReadAnotherSomethingElseNew(...);
        //    if (archive.ReadErrorOccured) break;
        //  }
        //}

        break;
      }

      // If BinaryArchiveWriter.BeginRead3dmChunk returns true,
      // then BinaryArchiveWriter.EndRead3dmChunk must be called
      // even if a write operation failed.
      rc = archive.EndRead3dmChunk(true);

      if (!rc)
      {
        Debug.WriteLine("Error writing document user data.");
      }
    }
  }

}